#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4 nu


import json
import types
from functools import wraps, partial

import django
from django.http import HttpResponse
from django.conf.urls import include, url as addurl

from utils import HttpResponseException


__all__ = ["view", "routing"]


def render_if(self, render_to=None, condition=lambda: False):
    """
        Render this view instead of the previous

        This function meant to be bound as a method to any fonction decorated
        with func_name.render_if, func_name being a function you just
        defined before, and decorated with @view.

        @view(render_to='template')
        def my_view(request):
            ...
            return context

        @my_view.render_if(condition=lambda r: r. user.is_authenticated())
        def my_conditional_view(request, context):
            ...

        my_view will always be executed, and should return a dictionary. if
        the user is authenticated, my_conditional_view is called, get the
        dictionary as a context, and should return a dictionary. If not,
        the context returned from my_view will be used directly.

        In any case, the context is rendered to the render_to template.

    """
    def decorator(func):
        self.conditional_calls.append((condition, func, render_to))
        return func
    return decorator

# Thers
render_if_ajax = partial(render_if, condition=lambda r, *a, **k: r.is_ajax())
render_if_get = partial(render_if, condition=lambda r, *a, **k: r.method == 'GET')
render_if_post = partial(render_if, condition=lambda r, *a, **k: r.method == 'POST')


def view(render_to=None):
    """
        Decorate a view to allow it to return only a dictionary and be rendered
        to either a template or json.

        @view(render_to="template"):
        def my_view(request):
            ...
            return {....}

        The returned dict will be used as a context, and rendered with
        the given template and RequestContext as an instance.

        @view(render_to="json"):
        def my_view(request):
            ...
            return {....}

        The returned dict will be used as a context, and rendered as json.

        The view will also gain new attributes that you can use as
        decorators to declare alternative function to execute after the view:

        @view(render_to='user.html'):
        def user_view(request, id)
            ...
            return {'users': users}


        @user_view.ajax(render_to='json')
        def ajax_user_view(request, id, context):
            ...
            return context

        ajax_user_view will be called only if it's an ajax request. It will
        be passed the result of user_view as a context, and it should return
        a dictionary which will be rendered as json.
    """

    def decorator(func):

        func.conditional_calls = []

        func.ajax = types.MethodType(render_if_ajax, func)
        func.get = types.MethodType(render_if_get, func)
        func.post = types.MethodType(render_if_post, func)
        func.render_if = types.MethodType(render_if, func)

        @wraps(func)
        def wrapper(request, *args, **kwargs):

            try:
                for test, view, rendering in func.conditional_calls:
                    if test(request, *args, **kwargs):
                        response = view(request,
                                         context=func(request, *args, **kwargs),
                                        *args, **kwargs)
                        break

                else:
                    response, rendering = func(request, *args, **kwargs), render_to

                rendering = rendering or render_to

                if rendering and not isinstance(response, HttpResponse):

                    if rendering == 'json':
                        return HttpResponse(json.dumps(response),
                                            mimetype="application/json")
                    if rendering == 'raw':
                        return HttpResponse(response)

                    return HttpResponse(response, content_type=rendering)

                return response
            except HttpResponseException as e:
                return e

        return wrapper

    return decorator


def routing(root=""):
    """
        Return a url patterns list that Django can use for routing, and
        a url decorator that adds any view as a route to this list.


        url, urlpatterns = routing()

        @url(r'/home/')
        def view(request):
            ...

        @url(r'/thing/(?P<pk>\d+)/$', name="thingy")
        def other_view(request, pk):
            ...

    """

    urlpatterns = UrlList()

    def url(regex, kwargs=None, name=None, prefix=''):

        def decorator(func):

            urlpatterns.append(
                addurl(regex, func, kwargs, name or func.__name__, prefix),
            )

            return func

        return decorator

    def http403(func):
        django.conf.urls.handler403 = func
        return func
    url.http403 = http403

    def http404(func):
        django.conf.urls.handler404 = func
        return func
    url.http404 = http404

    def http405(func):
        django.conf.urls.handler405 = func
        return func
    url.http405 = http405

    return url, urlpatterns


class UrlList(list):
    """
        Sublass list to allow shortcuts to add urls to this pattern.
    """

    admin_added = False


    def add_url(self, regex, func, kwargs=None, name="", prefix=""):
        self.append(addurl(regex, func, kwargs, name, prefix))


    def include(self, regex, module, name="", prefix=""):
        self.add_url(regex, include(module), name=name, prefix=prefix)


    def add_admin(self, url):

        from django.contrib import admin

        if not UrlList.admin_added:
            admin.autodiscover()

        self.include(url, admin.site.urls, 'admin')

        UrlList.admin_added = True



#!/usr/bin/env python
# -*- coding= UTF-8 -*-

"""
    Delete all sessions
"""

from optparse import make_option

from django.core.management.base import BaseCommand
from django.contrib.sessions.models import Session


class Command(BaseCommand):

    help = 'Delete all sessions from the server'

    option_list = BaseCommand.option_list + (

        make_option('--no-confirm',
            action='store_true',
            dest='no_confirm',
            default=False,
            help=u"Don't ask for confirmation"),

    )

    def handle(self, *args, **options):

        total = Session.objects.all().count()

        if not options['no_confirm']:
            confirm = raw_input('This will delete all %s sessions. Are you sure ? [y/N]\n' % total)
            if confirm.lower() not in ('y', 'yes'):
                return

        Session.objects.all().delete()
        print '%s sessions deleted' % total



#!/usr/bin/env python
# -*- coding: utf-8 -*-


"""
    Generate random usernames in
"""

import random

from .names import names as default_names


class NameGenerator(object):

    def __init__(self, names=None):
        self.names = names or default_names


    def __call__(self):
        return self.names.pop(random.randrange(len(self.names)))

    def __iter__(self):
        while self.names:
            yield self()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4 nu

import re
import random

from django.conf import settings
from django.views.static import serve
from django.contrib.staticfiles.views import serve as serve_static
from django.shortcuts import redirect, render

from django.contrib.auth.models import User

from namegen.namegen import NameGenerator

from utils import setting


class ForceSuperUserMiddleWare(object):
    """
        Developpement middleware forcing login with a super user so you
        don't have to login or worry about access rights.
    """

    def process_request(self, request):

        request.user = User.objects.filter(is_superuser=True)[0]


class StaticServe(object):
    """
        Django middleware for serving static files instead of using urls.py.

        It serves them wether you are set DEBUG or not, so put it into
        a separate settings file to activate it at will.
    """

    # STATIC_URL must be defined at least
    static_url = settings.STATIC_URL.rstrip('/')

    # try to get MEDIA_URL
    media_url = setting('MEDIA_URL', '').rstrip('/')

    # try to get MEDIA_URL
    admin_url = setting('ADMIN_MEDIA_PREFIX', '').rstrip('/')

    media_regex = re.compile(r'^%s/(?P<path>.*)$' % media_url)
    static_regex = re.compile(r'^%s/(?P<path>.*)$' % static_url)
    admin_regex = re.compile(r'^%s/(?P<path>.*)$' % admin_url)

    # IF not MEDIA_ROOT is defined, we supposed it's the same as the
    # STATIC_ROOT
    MEDIA_ROOT = setting('MEDIA_ROOT') or setting('STATIC_ROOT')
    ADMIN_ROOT = setting('ADMIN_MEDIA_PREFIX') or setting('STATIC_ROOT')


    def process_request(self, request):

        protocol = 'http' + ('', 's')[request.is_secure()]
        prefix = protocol + '://' + request.META['HTTP_HOST']
        abspath = prefix + request.path

        if self.media_url:
            path = abspath if prefix in self.media_url else request.path
            match = self.media_regex.search(path)
            if match:
                return serve(request, match.group(1), self.MEDIA_ROOT)

        if self.admin_url:
            path = abspath if prefix in self.admin_url else request.path
            match = self.admin_regex.search(path)
            if match:
                return serve(request, match.group(1), self.ADMIN_ROOT)

        path = abspath if prefix in self.static_url else request.path
        match = self.static_regex.search(path)
        if match:


class AutoLogNewUser(object):


    CALLBACK = setting('AUTOLOGNEWUSER_CALLBAK', None)


    def process_request(self, request):


        if 'django-quicky-test-cookie' in request.path:

            if not request.session.test_cookie_worked():
                return render(request, 'django_quicky/no_cookies.html',
                              {'next': request.GET.get('next', '/')})

            request.session.delete_test_cookie()

            first_name = iter(NameGenerator()).next().title()
            username = "%s%s" % (first_name, random.randint(10, 100))
            user = User.objects.create(username=username,
                                       first_name=first_name)
            request.session['django-quicky:user_id'] = user.pk
            next = request.GET.get('next', '/')
            if self.CALLBACK:
                res = self.CALLBACK(request)
            return redirect(res or next)

        if not request.user.is_authenticated():

            user_id = request.session.get('django-quicky:user_id', None)

            if not user_id:

                request.session.set_test_cookie()
                return redirect('/django-quicky-test-cookie/?next=%s' % request.path)

            request.user = User.objects.get(pk=user_id)





#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4 nu


import imp
import os
import sys

from django.http import HttpResponse
from django.core.management import setup_environ


__all__ = ['HttpResponseException', 'setting', 'get_client_ip', 'load_config']


class HttpResponseException(HttpResponse, Exception):
    pass


def setting(name, default=None):
    """
        Gets settings from django.conf if exists, returns default value otherwise

        Example:

        DEBUG = setting('DEBUG', False)
    """
    from django.conf import settings
    return getattr(settings, name, default)


def get_client_ip(request):
    """
        Return the client IP address as a string.
    """
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        return x_forwarded_for.split(',')[0]

    return request.META.get('REMOTE_ADDR')


def load_config(path, starting_point='.', settings_module='settings'):
    """
        Add the settings directory to the sys.path, import the settings and
        configure django with it.

        You can path an absolute or a relative path to it.

        If you choose to use a relative path, it will be relative to
        `starting_point` by default, which is set to '.'.

        You may want to set it to something like __file__ (the basename will
        be stripped, and the current file's parent directory will be used
        as a starting point, which is probably what you expect in the
        first place).

        :example:

        >>> load_config('../../settings.py', __file__)
    """

    if not os.path.isabs(path):

        if os.path.isfile(starting_point):
            starting_point = os.path.dirname(starting_point)

        path = os.path.join(starting_point, path)

    path = os.path.realpath(os.path.expandvars(os.path.expanduser(path)))

    if os.path.isfile(path):
        module = os.path.splitext(os.path.basename(path))[0]
        path = os.path.dirname(path)
    else:
        module = os.path.environ.get('DJANGO_SETTINGS_MODULE', settings_module)

    sys.path.append(path)

    f, filename, desc = imp.find_module(module, [path])
    project = imp.load_module(module, f, filename, desc)
    setup_environ(project)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4 nu


"""
    Part of the code is borrowed from django-annoying.

    https://bitbucket.org/offline/django-annoying/wiki/Home
"""

from django.db import models

from django.db.models import OneToOneField
from django.db.models.fields.related import SingleRelatedObjectDescriptor

try:
    from south.modelsinspector import add_introspection_rules
except ImportError:
    add_introspection_rules = lambda x: x


class AutoSingleRelatedObjectDescriptor(SingleRelatedObjectDescriptor):
    def __get__(self, instance, instance_type=None):
        try:
            return super(AutoSingleRelatedObjectDescriptor, self).__get__(instance, instance_type)
        except self.related.model.DoesNotExist:
            obj = self.related.model(**{self.related.field.name: instance})
            obj.save()
            return obj


class AutoOneToOneField(OneToOneField):
    '''
        OneToOneField creates related object on first call if it doesnt exists yet.
        Use it instead of original OneToOne field.

        example:
        class MyProfile(models.Model):
        user = AutoOneToOneField(User, primary_key=True)
        home_page = models.URLField(max_length=255)
        icq = models.CharField(max_length=255)
    '''
    def contribute_to_related_class(self, cls, related):
        setattr(cls, related.get_accessor_name(), AutoSingleRelatedObjectDescriptor(related))



class IntegerRangeField(models.IntegerField):

    """
        Equvalent of the django Integer Field but with min and max value.
    """

    def __init__(self, verbose_name=None, name=None,
                 min_value=None, max_value=None, **kwargs):
        self.min_value, self.max_value = min_value, max_value
        models.IntegerField.__init__(self, verbose_name, name, **kwargs)


    def formfield(self, **kwargs):
        defaults = {'min_value': self.min_value, 'max_value':self.max_value}
        defaults.update(kwargs)
        return super(IntegerRangeField, self).formfield(**defaults)



# if South is installed, provide introspection rules for it's migration
# see: http://south.aeracode.org/docs/tutorial/part4.html#tutorial-part-4
add_introspection_rules([
    (
        [IntegerRangeField],
        [],
        {
            "min_value": ["min_value", {"default": None}],
            "max_value": ["max_value", {"default": None}],
        },
    ),
], ["^libs\.models\.IntegerRangeField"])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4 nu


VERSION = __version__ = "0.6"

from decorators import view, routing
from utils import setting, load_config
from models import get_object_or_none

